from setuptools import setup

setup(name='bgno',
      version='0.1',
      description='Gaussian and Binomial distributions',
      packages=['bgno'],
      author_email='arturgugasian@gmail.com',
      zip_safe=False)